﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Transactions;
using Common;
using DAL;
using IBLL;

namespace BLL
{
    public class InformationsBLL : IInformationsBLL, IDisposable
    {
        /// <summary>
        /// 私有的数据访问上下文
        /// </summary>
        protected SysEntities db;

        public InformationsBLL()
            : this(new SysPersonBLL())
        {
        }

        ISysPersonBLL _iSysPersonBll;
        /// <summary>
        /// 构造函数，默认加载数据访问上下文
        /// </summary>
        public InformationsBLL(SysPersonBLL sysPersonBll)
        {
            _iSysPersonBll = sysPersonBll;
            db = new SysEntities();
        }

        /// <summary>
        /// 验证信息标题是否存在
        /// </summary>
        /// <param name="entity">信息实体</param>
        /// <returns></returns>
        public bool CheckTitle(Informations entity)
        {
            return db.Informations.Any(o=>o.Title==entity.Title.Trim()
                &&o.CreatePersonId==entity.CreatePersonId &&o.Id!=entity.Id);
        }


        /// <summary>
        /// 查询的数据
        /// </summary>
        /// <param name="id">额外的参数</param>
        /// <param name="page">页码</param>
        /// <param name="rows">每页显示的行数</param>
        /// <param name="order">排序字段</param>
        /// <param name="sort">升序asc（默认）还是降序desc</param>
        /// <param name="search">查询条件</param>
        /// <param name="total">结果集的总数</param>
        /// <returns>结果集</returns>
        public List<Informations> GetByParam(string sysPersonId, string id, int page, int rows, string order,
            string sort, string search, ref int total)
        {
            using (var repository = new InformationsRepository())
            {
                IQueryable<Informations> queryData=null;
                var sysRole = _iSysPersonBll.GetRefSysRole(sysPersonId).FirstOrDefault();
                if (sysRole.Power == 1 || sysRole.Power == 2)
                {
                    queryData = repository.DaoChuData(db, order, sort, search);
                }
                else
                {
                    queryData = repository.DaoChuData(db, order, sort, search)
                        .Where(o => o.CreatePersonId == sysPersonId);
                }
                total = queryData.Count();
                if (total > 0)
                {
                    if (page <= 1)
                    {
                        queryData = queryData.Take(rows);
                    }
                    else
                    {
                        queryData = queryData.Skip((page - 1) * rows).Take(rows);
                    }
                    foreach (var item in queryData)
                    {
                        if (item.FeedbackTemplate != null)
                        {
                            item.FeedbackTemplateId = string.Empty;
                            foreach (var it in item.FeedbackTemplate)
                            {
                                item.FeedbackTemplateId += it.Name + ' ';
                            }
                        }
                        if (item.Member != null)
                        {
                            item.MemberId = string.Empty;
                            foreach (var it in item.Member)
                            {
                                item.MemberId += it.Name + ' ';
                            }
                        }
                    }
                }
                return queryData.ToList();
            }
        }

        /// <summary>
        /// 添加信息接收对象和使用的反馈模板
        /// </summary>
        /// <param name="db"></param>
        /// <param name="entity"></param>
        private void AddInfoMemberAndFeedbackTemplate( SysEntities db, Informations entity)
        {

                foreach (string item in entity.FeedbackTemplateId.GetIdSort())
                {
                    db = new SysEntities();
                    string sql = @"INSERT INTO InformationFeedbackTemplate(InformationId,FeedbackTemplateId)
                                VALUES(@InformationId,@FeedbackTemplateId)";
                    var args = new DbParameter[] 
                        { 
                             new SqlParameter { ParameterName = "InformationId", Value =entity.Id},
                             new SqlParameter { ParameterName = "FeedbackTemplateId", Value =item},
                        };
                    db.ExecuteStoreCommand(sql, args);
                }
                foreach (string item in entity.MemberId.GetIdSort())
                {
                    db = new SysEntities();
                    string sql = @"INSERT INTO InformationMember(InformationId,MemberId)
                                VALUES(@InformationId,@MemberId)
                                INSERT INTO MemberInformations(InformationsId,MemberId)
                                                                VALUES(@InformationId,@MemberId)";
                    var args = new DbParameter[] 
                        { 
                             new SqlParameter { ParameterName = "InformationId", Value =entity.Id},
                             new SqlParameter { ParameterName = "MemberId", Value =item},
                        };
                    db.ExecuteStoreCommand(sql, args);
                }
        }

        public bool Create(ref ValidationErrors validationErrors, SysEntities db, Informations entity)
        {
            using (var repository = new InformationsRepository())
            {
                try
                {
                    if (CheckTitle(entity))
                    {
                        validationErrors.Add("： 使用标题不能重复，请重新输入标题！");
                        return false;
                    }
                    else if (entity.FeedbackTemplateId.GetIdSort().Count==0)
                    {
                        validationErrors.Add("： 态度反馈模板不能为空！");
                        return false;
                    }
                    else if (entity.MemberId.GetIdSort().Count==0)
                    {
                        validationErrors.Add("： 会员不能为空！");
                        return false;
                    }
                    if (entity.FeedbackTemplateId.GetIdSort().Count > 4)
                    {
                        validationErrors.Add("： 态度反馈模板最多只能设定4个！");
                        return false;
                    }
                    else
                    {
                        repository.Create(entity);
                        AddInfoMemberAndFeedbackTemplate(db, entity);
                        return true;
                    }
                }
                catch (Exception ex)
                {
                    validationErrors.Add(ex.Message);
                    ExceptionsHander.WriteExceptions(ex);
                }
                return false;
            }
        }

        /// <summary>
        /// 添加信息
        /// </summary>
        /// <param name="validationErrors"></param>
        /// <param name="entity"></param>
        /// <returns></returns>
        public bool Create(ref ValidationErrors validationErrors, Informations entity)
        {
            try
            {
                using (TransactionScope transactionScope = new TransactionScope())
                {
                    if (Create(ref validationErrors, db, entity))
                    {
                        transactionScope.Complete();
                        return true;
                    }
                    else
                    {
                        Transaction.Current.Rollback();
                    }
                }
            }
            catch (Exception ex)
            {
                validationErrors.Add(ex.Message);
                ExceptionsHander.WriteExceptions(ex);
            }
            return false;
        }

        public bool Delete(ref ValidationErrors validationErrors, string id)
        {
            using (var repository = new InformationsRepository())
            {
                try
                {
                    return repository.Delete(id) == 1;
                }
                catch (Exception ex)
                {
                    validationErrors.Add(ex.Message);
                    ExceptionsHander.WriteExceptions(ex);
                }

                return false;
            }
        }

        /// <summary>
        /// 删除信息集合
        /// </summary>
        /// <param name="validationErrors">返回的错误信息</param>
        /// <param name="deleteCollection">主键的信息</param>
        /// <returns></returns>    
        public bool DeleteCollection(ref ValidationErrors validationErrors, string[] deleteCollection)
        {
            using (var repository = new InformationsRepository())
            {
                try
                {
                    if (deleteCollection != null)
                    {

                        using (TransactionScope transactionScope = new TransactionScope())
                        {
                            repository.Delete(db, deleteCollection);
                            if (deleteCollection.Length == repository.Save(db))
                            {
                                transactionScope.Complete();
                                return true;
                            }
                            else
                            {
                                Transaction.Current.Rollback();
                            }
                        }

                    }
                }
                catch (Exception ex)
                {
                    validationErrors.Add(ex.Message);
                    ExceptionsHander.WriteExceptions(ex);
                }
                return false;
            }
        }

        /// <summary>
        /// 编辑一个信息
        /// </summary>
        /// <param name="validationErrors">返回的错误信息</param>
        /// <param name="entity">一个数据字典</param>
        /// <returns></returns>
        public bool Edit(ref ValidationErrors validationErrors,SysEntities db, Informations entity)
        {
            using (var repository = new InformationsRepository())
            {
                try
                {
                    if (CheckTitle(entity))
                    {
                        validationErrors.Add("： 使用标题不能重复，请重新输入标题！");
                        return false;
                    }
                    else if (entity.FeedbackTemplateId.GetIdSort().Count == 0)
                    {
                        validationErrors.Add("： 态度反馈模板不能为空！");
                        return false;
                    }
                    else if (entity.MemberId.GetIdSort().Count == 0)
                    {
                        validationErrors.Add("： 会员不能为空！");
                        return false;
                    }
                    else if (entity.FeedbackTemplateId.GetIdSort().Count > 4)
                    {
                        validationErrors.Add("： 态度反馈模板最多只能设定4个！");
                        return false;
                    }
                    else
                    {
                        repository.Edit(db, entity);
                        repository.Save(db);
                        string deleteSql =
                                          @"DELETE FROM InformationFeedbackTemplate
                                        WHERE InformationId=@InformationId
                                        DELETE FROM InformationMember
                                        WHERE InformationId=@InformationId
                                        DELETE FROM MemberInformations
                                        WHERE InformationsId=@InformationId";
                        var deleteArg = new DbParameter[] 
                        { new SqlParameter { ParameterName = "InformationId", Value = entity.Id } };
                        db.ExecuteStoreCommand(deleteSql, deleteArg);
                        AddInfoMemberAndFeedbackTemplate( db, entity);
                        return true;
                    }

                   
                }
                catch (Exception ex)
                {
                    validationErrors.Add(ex.Message);
                    ExceptionsHander.WriteExceptions(ex);
                }
                return false;
            }
        }

        /// <summary>
        /// 编辑一个信息
        /// </summary>
        /// <param name="validationErrors">返回的错误信息</param>
        /// <param name="entity">一个信息</param>
        /// <returns>是否编辑成功</returns>
        public bool Edit(ref ValidationErrors validationErrors, Informations entity)
        {
            using (var repository = new MemberGroupRepository())
            {
                try
                {
                    SysEntities newDb = new SysEntities();
                    using (TransactionScope transactionScope = new TransactionScope())
                    {
                        if (Edit(ref validationErrors, newDb, entity))
                        {
                            transactionScope.Complete();
                            return true;
                        }
                        else
                        {
                            Transaction.Current.Rollback();
                        }
                    }
                }
                catch (Exception ex)
                {
                    validationErrors.Add(ex.Message);
                    ExceptionsHander.WriteExceptions(ex);
                }
                return false;
            }
        }

        /// <summary>
        /// 获取所有信息
        /// </summary>
        /// <returns></returns>
        public List<Informations> GetAll()
        {
            using (var repository = new InformationsRepository())
            {
                return repository.GetAll(db).ToList();
            }
        }

        /// <summary>
        /// 根据用户Id获取他其所发的信息
        /// </summary>
        /// <param name="personId">用户Id</param>
        /// <returns></returns>
        public List<Informations> GetInformationsBySysPersonId(string personId)
        {
            return db.Informations.Where(o => o.CreatePersonId == personId).ToList();
        }

        /// <summary>
        /// 根据主键获取一个信息
        /// </summary>
        /// <param name="id">信息的主键</param>
        /// <returns>一个商家信息</returns>
        public Informations GetById(string id)
        {
            using (var repository = new InformationsRepository())
            {
                return repository.GetById(db, id);
            }
        }

        /// <summary>
        /// 信息反馈统计
        /// </summary>
        /// <param name="informationId">信息编号</param>
        /// <returns></returns>
        public InformationFeedbackReport GetInformationFeedbackReport(string informationId)
        {
            InformationFeedbackReport report = new InformationFeedbackReport();

            using (var repository = new InformationsRepository())
            {
                try
                {
                    string sql = @"SELECT Title,CreateTime 
                                    FROM dbo.Informations 
                                    WHERE Id=@InformationId

                            select Name, t.FeedbackTemplateId, amount = case t.Id when NULL
							then 0 
						else 
							COUNT(t.Id)
				end
				from
            (select ft.Name,ift.FeedbackTemplateId ,ifb.Id				
            from InformationFeedbackTemplate as ift
            left join InformationFeedback as ifb
             on ifb.FeedbackTemplateId = ift.FeedbackTemplateId      
            and ifb.InformationsId = ift.InformationId    
            left join FeedbackTemplate as ft 
            on ft.Id = ift.FeedbackTemplateId
            where ift.InformationId = @InformationId) as t
            group by Name,FeedbackTemplateId

                            SELECT M.Name,IFB.Question,CreateTime  
                            FROM InformationFeedback IFB
                            INNER JOIN dbo.Member M 
                            ON IFB.MemberId = M.Id
                            WHERE InformationsId=@InformationId  AND IFB.Question IS NOT NULL

                            SELECT COUNT(*)AS Total 
                            FROM InformationMember 
                            WHERE InformationId=@InformationId";
                    SqlCommand scmd = new SqlCommand(sql);

                    scmd.Parameters.AddWithValue("@InformationId", informationId);
                    var dataSources = ObjectQueryExtensions.ExceuteEntityProcReturnDataset(db, scmd);
                    var dataSource1 = dataSources.Tables[0];
                    var dataSource2 = dataSources.Tables[1];
                    var dataSource3 = dataSources.Tables[2];
                    var dataSource4 = dataSources.Tables[3];
                    for (int i = 0; i < dataSource1.Rows.Count; i++)
                    {
                        report.Title = dataSource1.Rows[i]["Title"].ToString();
                        report.SendTime = Convert.ToDateTime(dataSource1.Rows[i]["CreateTime"]);
                    }
                    foreach (DataRow item in dataSource2.Rows)
                    {
                        var f = new FeedbackTemplateStatistics();
                        f.FeedbackTemplateId = item["FeedbackTemplateId"].ToString();
                        f.FeedbackTemplateName = item["Name"].ToString();
                        f.FeedbackTemplateFeedbackAmount = Convert.ToInt32(item["amount"]);
                        report.FeedbackMemberAmount += f.FeedbackTemplateFeedbackAmount;
                        report.FeedbackTemplateStatisticsList.Add(f);
                    }

                    foreach (DataRow item in dataSource3.Rows)
                    {
                        var m = new FeedbackTemlateMembers();
                        m.MemberName = item["Name"].ToString();
                        m.CreateTime = Convert.ToDateTime(item["CreateTime"]);
                        m.FeedbackContent = item["Question"].ToString();
                        report.FeedbackTemlateMembersList.Add(m);
                    }

                    foreach (DataRow item in dataSource4.Rows)
                    {
                        report.ReceiveMemberAmount = Convert.ToInt32(item["Total"]);
                    }
                    if (report!=null)
                    {
                        return report;
                    }

                    return null;
                }
                catch (Exception)
                {
                    return null;
                }

            }
        }
       
        public void Dispose()
        {

        }
    }
}
